/* global Bdf,Ficheform,$$,Fql,Fapi */

Ficheform.CorpusInclude = {};

Ficheform.CorpusInclude.init = function () {
    Bdf.Api.init();
    Bdf.$({ficheformEntry: "corpus-include", ficheformMode: "check"}).each(function (index, element) {
        _initCheckCorpus(element);
    });
    Bdf.$({ficheformEntry: "corpus-include", ficheformMode: "table"}).each(function (index, element) {
        _initTableCorpus(element);
    });
    _initLiage();
   
    
    function _initCheckCorpus(element) {
        let hereMap = new Map();
        let entryKey = element.dataset.ficheformKey;
        let $entry = $(element);
        let withPoids = (Bdf.$($entry, {ficheformRole: "poids"}).length > 0); // à corriger un jour, peut-être
        _initDirectInput(element);
        _initSearchArea($entry, Bdf.getSubsetName($entry), hereMap, __addChoice);
        if (withPoids) {
            Ficheform.initPoidsInput(Bdf.$children($entry, {ficheformRole: "cell-enumeration"}));
        }
        let $choices = Bdf.$($entry, {ficheformRole: "item-choices"});
        Bdf.$($choices, {ficheformRole: "item-choice"}).each(function (index, element) {
           __initChoice(element);
        });
        
        function __initChoice(element) {
            let ficheId = element.dataset.ficheformItemId;
            hereMap.set(parseInt(ficheId), true);
            let genId = Bdf.generateId();
            Bdf.$(element, {_element: "label"}).after(Bdf.render("ficheform:include/choicebuttons", {
                genId: genId,
                noTab: Ficheform.getOption("noTab")
            }));
            Ficheform.initOrderButtons(genId, element);
        }
        
        function __addChoice(subsetName, id) {
            Bdf.api.loadFiche({
                corpus: subsetName,
                id: id,
                requestParameters: {
                    properties: "special:phrases"
                },
                callback: function (data) {
                    let fiche = data.fiche;
                    _showEnumeration($entry);
                    $choices.append(Bdf.render("ficheform:include/itemchoice", {
                        itemId: id,
                        checkboxName: entryKey,
                        checkboxValue: id,
                        checkboxId: Bdf.generateId(),
                        withPoids: withPoids,
                        number: fiche.properties.number,
                        title: fiche.title
                    }));
                    __initChoice(Bdf.$($choices, {ficheformItemId: id})[0]);
                    Ficheform.signalChange(true);
                }
           });
        }
    }
    
    function _initTableCorpus(element) {
        let hereMap = new Map();
        let $entry = $(element);
        _initDirectInput(element);
        let entryKey = element.dataset.ficheformKey;
        let subsetName =  Bdf.getSubsetName($entry);
        let parentCorpus = Ficheform.getCurrentCorpus($entry);
        _initSearchArea($entry, subsetName, hereMap, __addRow);
        let $ficheRows = Bdf.$($entry, {ficheformRole: "fiche-rows"});
        let colNameArray = $ficheRows[0].dataset.ficheformColDefList.split(",");
        Bdf.$($ficheRows, {ficheformRole: "fiche-row"}).each(function (index, element) {
           __initRow(element); 
        });
        
        function __initRow(tr) {
            let ficheId = tr.dataset.ficheformItemId;
            hereMap.set(parseInt(ficheId), true);
            let genId = Bdf.generateId();
            Bdf.$(tr, {ficheformRole: "fiche-row-buttons"}).html(Bdf.render("ficheform:include/rowbuttons", {
                genId: genId,
                noTab: Ficheform.getOption("noTab")
               }));
            Ficheform.initOrderButtons(genId, tr);
            $$(genId, "edit").click(function () {
                Bdf.Overlay.showFicheEditor(subsetName, ficheId, {
                    doneCallback: function (subsetName, id) {
                        Bdf.api.loadFiche({
                            corpus: subsetName,
                            id: id,
                            requestParameters: {
                                properties: "special:table",
                                contextcorpus: parentCorpus
                            },
                            callback: function (data) {
                                let fiche = data.fiche;
                                let cellArray = [];
                                for(let colName of colNameArray) {
                                    if (fiche.properties.hasOwnProperty(colName)) {
                                        cellArray.push(fiche.properties[colName]);
                                    } else {
                                        cellArray.push("");
                                    }
                                }
                                Bdf.$($ficheRows, {ficheformItemId: id}).children().each(function (index, element) {
                                    if (index > 0) {
                                        $(element).html(cellArray[index - 1]);
                                    }
                                });
                            }
                        });
                    }
                });
            });
            $$(genId, "remove").click(function () {
                Bdf.$(tr, {_element: "input", _type: "checkbox"}).prop("checked", false);
                $(tr).addClass("ficheform-fichetable-Remove");
                $$(genId, "remove").addClass("hidden");
                $$(genId, "add").removeClass("hidden");
            });
            $$(genId, "add").click(function () {
                Bdf.$(tr, {_element: "input", _type: "checkbox"}).prop("checked", true);
                $(tr).removeClass("ficheform-fichetable-Remove");
                $$(genId, "remove").removeClass("hidden");
                $$(genId, "add").addClass("hidden");
            });
        }
        
        function __addRow(subsetName, id) {
            Bdf.api.loadFiche({
                corpus: subsetName,
                id: id,
                requestParameters: {
                    properties: "special:table",
                    contextcorpus: parentCorpus
                },
                callback: function (data) {
                    let fiche = data.fiche;
                    let cellArray = [];
                    for(let colName of colNameArray) {
                        if (fiche.properties.hasOwnProperty(colName)) {
                            cellArray.push(fiche.properties[colName]);
                        } else {
                            cellArray.push("");
                        }
                    }
                    _showEnumeration($entry);
                    $ficheRows.append(Bdf.render("ficheform:include/ficherow", {
                        itemId: id,
                        checkboxName: entryKey,
                        checkboxValue: id,
                        cellArray: cellArray
                    }));
                    __initRow(Bdf.$($ficheRows, {ficheformItemId: id})[0]);
                    Ficheform.signalChange(true);
                }
           });
        }
    }
    
    function _initLiage() {
        Bdf.$({ficheformEntry: "liage"}).each(function (index, element) {
            let $entry = $(element);
            let entryKey = element.dataset.ficheformKey;
            let $directInput = Bdf.$($entry, {ficheformRole: "fiche-directinput"});
            let inputId = Bdf.$($directInput, {ficheformRole: "textinput"})[0].id;
            let buttonId = Bdf.generateId();
            $directInput.append(Bdf.render("ficheform:pioche", {
                buttonId: buttonId,
                action: "action-ficheform-Liage",
                locKey: "_ link.pioche.fiche_all",
                noTab: Ficheform.getOption("noTab")
            }));
            $$(buttonId).click(function () {
                Ficheform.CorpusInclude.pioche(entryKey, "", inputId, false);
            });
            Ficheform.initPoidsInput($entry);
        });
    }
    
    function _showEnumeration($entry) {
        Bdf.$children($entry, {ficheformRole: "cell-enumeration"}).removeClass("hidden");
    }
    
    function _initDirectInput(element) {
        let $entry = $(element);
        let $directInput = Bdf.$($entry, {ficheformRole: "fiche-directinput"});
        let entryKey = element.dataset.ficheformKey;
        let subsetName =  Bdf.getSubsetName($entry);
        let inputId = Bdf.$($directInput, {ficheformRole: "textinput"})[0].id;
        let buttonId = Bdf.generateId();
        $directInput.append(Bdf.render("ficheform:pioche", {
            buttonId: buttonId,
            action: "action-ficheform-Fiche",
            locKey: "_ link.pioche.fiche_one",
            locValue: Bdf.Loc.get("subset:corpus_" + subsetName),
            noTab: Ficheform.getOption("noTab")
        }));
        $$(buttonId).click(function () {
            Ficheform.CorpusInclude.pioche(entryKey, subsetName, inputId, true);
        });        
    }
    
    function _initSearchArea($entry, corpusName, hereMap, addCallback) {
        let withCreate = Ficheform.ARGS.permissions.hasOwnProperty(corpusName);
        let searchInputId = Bdf.generateId();
        let createButtonId = (withCreate)?Bdf.generateId():0;
        let searchAnchorId = Bdf.generateId();
        let createTitle;
        if ((withCreate) && (Bdf.Loc.contains("newfiche:corpus_" + corpusName))) {
            createTitle = Bdf.Loc.get("newfiche:corpus_" + corpusName);
        }
        Bdf.$($entry, {ficheformRole: "searchcontainer"}).prepend(Bdf.render("ficheform:include/searcharea", {
            noTab: Ficheform.getOption("noTab"),
            createButtonId: createButtonId,
            searchInputId: searchInputId,
            withCreate: withCreate,
            createTitle: createTitle
        })).after(Bdf.render("ficheform:include/searchanchor", {
            searchAnchorId: searchAnchorId
        }));
        Ficheform.initSearchInput(searchInputId, searchAnchorId, function (input) {
            Ficheform.delayFunction(input.id, function () {
                let query = input.value.trim();
                if (query) {
                    __search(query);
                } else {
                    $$(searchAnchorId).html("");
                }
            });
        });
        Bdf.onClick($$(searchAnchorId), {action: "itemadd"}, function () {
            addCallback(corpusName, this.dataset.id);
            $$(searchAnchorId).html("");
            $$(searchInputId).val("");
            return false;
        });
        $$(createButtonId).click(function () {
            Bdf.Overlay.showFicheEditor(corpusName, -1, {
                doneCallback: addCallback
            });
        });
        
        function __search(query) {
            Bdf.api.loadFicheArray({
                ficheQuery: {
                    corpus: corpusName,
                    content: {
                        scope: Fql.SCOPE_TITRE,
                        q: query,
                        type: Fql.QTYPE_SIMPLE
                    }
                },
                requestParameters: {
                    sort: "id-desc",
                    limit: 50
                },
                callback: function (data) {
                    let ficheArray = data.array;
                    if (ficheArray.length === 0) {
                        $$(searchAnchorId).html(Bdf.render("ficheform:noresult"));
                    } else {
                        let first = true;
                        for(let fiche of ficheArray) {
                            if (hereMap.has(fiche.id)) {
                                fiche._here = true;
                            } else if (first) {
                                fiche._selected = first;
                                first = false;
                            }
                        }
                        $$(searchAnchorId).html(Bdf.render("ficheform:include/searchlist", {array: ficheArray}));
                    }
                }
            });
        }
    }

};

Ficheform.CorpusInclude.pioche = function (entryKey, subsets, inputId, withFicheCreationButton) {
    var $entry = Ficheform.$entry(entryKey);
    var limit = $entry.data("ficheformLimit");
    if (!limit) {
        limit = -1;
    }
    var url = "pioche?page=fiche&subsets=" + subsets + "&appelant=" + inputId + "&limit=" + limit;
    if (withFicheCreationButton) {
        url = url + "&buttons=fichecreation" ;
    }
    Ficheform.openDialog(url,"pioche_" + entryKey, 430,510);
};


Ficheform.CorpusInclude.showFicheCreation = function (inputId) {
    let $parent = Bdf.$parents($$(inputId), {subsetName: true});
    if ($parent.length > 0) {
        let subsetName = Bdf.getSubsetName($parent);
        Bdf.Overlay.showFicheEditor(subsetName, -1, {
            doneCallback: function (subsetName, id) {
                Bdf.appendValue(ID(inputId), id);
            }
        });
    }
};
